#include "ActionNeuralNetwork.h"
#include "State.h"
#include "Utility.h"

#include <cerrno>
#include <cfloat>
#include <cmath>


ActionNeuralNetwork::ActionNeuralNetwork( State* aStatePtr, const int numberOfHiddenNeurons )
  : ActionValueFunction( aStatePtr ),
    NeuralNetwork( aStatePtr, numberOfHiddenNeurons )
{
  INPUT_HIDDEN_ARRAY_SIZE = 0;
  HIDDEN_OUTPUT_ARRAY_SIZE = 0;
  INPUT_HIDDEN_OUTPUT_ARRAY_SIZE = 0;

  // den Funktionspointer initialisieren
  ValueFunction::minValue = -0.5;
  ValueFunction::maxValue = 0.5;
  initWeightsFunctionPtr = &getRandomValue;
}


ActionNeuralNetwork::ActionNeuralNetwork( const std::string& fileName )
  : ActionValueFunction( 0 ),
    NeuralNetwork()
{ 
  // Laden
  ActionValueFunction::load( fileName );
}


ActionNeuralNetwork::~ActionNeuralNetwork()
{
}


void ActionNeuralNetwork::create()
{
  INPUT_HIDDEN_ARRAY_SIZE = INPUT_LAYER_NEURONS * (HIDDEN_LAYER_NEURONS - 1) * ACTIONS; // -1 wegen Bias
  HIDDEN_OUTPUT_ARRAY_SIZE = HIDDEN_LAYER_NEURONS * OUTPUT_LAYER_NEURONS * ACTIONS;
  INPUT_HIDDEN_OUTPUT_ARRAY_SIZE = INPUT_LAYER_NEURONS * (HIDDEN_LAYER_NEURONS - 1) * OUTPUT_LAYER_NEURONS * ACTIONS; // -1 wegen Bias

  NeuralNetwork::allocateMemory();
  
  // init Skalierungsfaktoren
  for ( int i = 0; i < INPUT_LAYER_NEURONS - 1; ++i ) { // -1 wegen Bias
    scaleFactor[i] = 1.0;
  }

  // Bias setzen
  setBias( 1.0, 1.0 );
}


void ActionNeuralNetwork::initWeights( const double (*initFunctionPtr)() )
{
  NeuralNetwork::initWeights(initFunctionPtr);
}


void ActionNeuralNetwork::initEligibilityTraces()
{
  NeuralNetwork::initEligibilityTraces();
}


void ActionNeuralNetwork::updateWeights( const double ALPHA, const double DELTA )
{
  NeuralNetwork::updateWeights(ALPHA, DELTA, minWeightLimit, maxWeightLimit);
}


void ActionNeuralNetwork::updateEligibilityTraces( const double GAMMA, const double LAMBDA )
{
  NeuralNetwork::updateEligibilityTraces(GAMMA, LAMBDA);
}


void ActionNeuralNetwork::accumulatingTraces( const double VALUE )
{
  NeuralNetwork::accumulatingTraces(VALUE);
}


void ActionNeuralNetwork::replacingTraces( const double VALUE )
{
  NeuralNetwork::replacingTraces(VALUE);
}


const double ActionNeuralNetwork::getActionValue( const int actionIndex ) const
{
  if ( (actionIndex < 0) || (actionIndex >= ACTIONS) ) {
    RL_LOGERROR( "actionIndex[" << actionIndex<< "] out of bounds [ 0, " << ACTIONS << ")" );
    return 0.0;
  }
  return NeuralNetwork::getOutputValue(actionIndex);
}


void ActionNeuralNetwork::checkWeightLimit()
{
  NeuralNetwork::checkWeightLimit(minWeightLimit, maxWeightLimit);
}


const double ActionNeuralNetwork::getMinWeight() const
{
  return NeuralNetwork::getMinWeight();
}


const double ActionNeuralNetwork::getAvgWeight( const bool abs ) const
{
  return NeuralNetwork::getAvgWeight(abs);
}


const double ActionNeuralNetwork::getMaxWeight() const
{
  return NeuralNetwork::getMaxWeight();
}


void ActionNeuralNetwork::setState()
{
  NeuralNetwork::setState();
}


void ActionNeuralNetwork::storeState()
{
  NeuralNetwork::storeState();
}


void ActionNeuralNetwork::initEpisodeStateHistory()
{
  NeuralNetwork::initEpisodeStateHistory();
}


void ActionNeuralNetwork::valueEpisode( const double value )
{
  NeuralNetwork::valueEpisode(value);
}


void ActionNeuralNetwork::removeAction( const int actionIndex )
{
  RL_LOGERROR( "Removing actions isn't implemented yet for NeuralNetworks!" );
}


const int ActionNeuralNetwork::save( FILE* file ) const
{
  return NeuralNetwork::save( file );
}


const int ActionNeuralNetwork::load( FILE* file )
{
  // jetzt ist der StatePtr bereits geladen --> auch im LookupTable setzen
  NeuralNetwork::setStatePtr( ValueFunction::statePtr );
  return NeuralNetwork::load( file );
}
