#include "ActionTileCoding.h"
#include "ActionTilings.h"
#include "Log.h"

#include <cerrno>
#include <cfloat>


ActionTileCoding::ActionTileCoding( State* aStatePtr )
  : ActionValueFunction( aStatePtr ),
    TileCoding()
{
  // den Funktionspointer initialisieren
  value = 0.0;
  initWeightsFunctionPtr = &getValue;
}


ActionTileCoding::ActionTileCoding( const std::string& fileName )
  : ActionValueFunction( 0 ),
    TileCoding()
{
  // Laden
  ActionValueFunction::load( fileName );
}


ActionTileCoding::~ActionTileCoding()
{
}


void ActionTileCoding::initWeights( const double (*initFunctionPtr)() )
{
  TileCoding::initWeights( initFunctionPtr );
}


void ActionTileCoding::initEligibilityTraces()
{
  TileCoding::initEligibilityTraces();
}


void ActionTileCoding::updateWeights( const double ALPHA, const double DELTA )
{
  TileCoding::updateWeights( ALPHA, DELTA );
}


void ActionTileCoding::updateEligibilityTraces( const double GAMMA, const double LAMBDA )
{
  TileCoding::updateEligibilityTraces( GAMMA, LAMBDA );
}


void ActionTileCoding::registerTiling( ActionTilings* actionTilingPtr )
{
  actionTilingPtr->create( statePtr, ACTIONS );
  actionTilingPtr->initWeights( initWeightsFunctionPtr );
  tilingPtrList.push_back( actionTilingPtr );
  numberOfTilings += actionTilingPtr->getNumberOfTilings();
}


const double ActionTileCoding::getActionValue( const int actionIndex ) const
{
  double value = 0.0;
  for ( std::list< Tilings* >::const_iterator t = tilingPtrList.begin();
        t != tilingPtrList.end(); ++t )
  {
    value += static_cast< ActionTilings* >(*t)->getActionValue( actionIndex );
  }

  return value;
}


void ActionTileCoding::setActionValue( const int actionIndex, const double value )
{
  for ( std::list< Tilings* >::const_iterator t = tilingPtrList.begin();
        t != tilingPtrList.end(); ++t )
  {
    static_cast< ActionTilings* >(*t)->setActionValue( actionIndex, value );
  }
}


void ActionTileCoding::accumulatingTraces( const double VALUE )
{
  for ( std::list< Tilings* >::iterator t = tilingPtrList.begin();
        t != tilingPtrList.end(); ++t )
  {
    static_cast< ActionTilings* >(*t)->accumulatingTraces( VALUE, activeActionIndex );
  }

  if ( episodeStateHistory ) {
    for ( std::list< Tilings* >::const_iterator t = tilingPtrList.begin();
          t != tilingPtrList.end(); ++t )
    {
      static_cast< ActionTilings* >(*t)->storeEpisodeState( activeActionIndex );
    }
  }
}


void ActionTileCoding::replacingTraces( const double VALUE )
{
  for ( std::list< Tilings* >::iterator t = tilingPtrList.begin();
        t != tilingPtrList.end(); ++t )
  {
    static_cast< ActionTilings* >(*t)->replacingTraces( VALUE, activeActionIndex );
  }

  if ( episodeStateHistory ) {
    for ( std::list< Tilings* >::const_iterator t = tilingPtrList.begin();
          t != tilingPtrList.end(); ++t )
    {
      static_cast< ActionTilings* >(*t)->storeEpisodeState( activeActionIndex );
    }
  }
}


void ActionTileCoding::storeState()
{
  for ( std::list< Tilings* >::const_iterator t = tilingPtrList.begin();
        t != tilingPtrList.end(); ++t )
  {
    static_cast< ActionTilings* >(*t)->storeActiveTiles( activeActionIndex );
  }

  if ( episodeStateHistory ) {
    for ( std::list< Tilings* >::const_iterator t = tilingPtrList.begin();
          t != tilingPtrList.end(); ++t )
    {
      static_cast< ActionTilings* >(*t)->storeEpisodeState( activeActionIndex );
    }
  }
}


void ActionTileCoding::checkWeightLimit()
{
  // Grenzen der Gewichte ueberpruefen
  if ( (minWeightLimit > -1.0 * DBL_MAX) || (maxWeightLimit < DBL_MAX) ) {
    for ( std::list< Tilings* >::iterator t = tilingPtrList.begin();
          t != tilingPtrList.end(); ++t )
    {
      (*t)->checkWeightLimit( minWeightLimit, maxWeightLimit );
    }
  }
}


const double ActionTileCoding::getMinWeight() const
{
  return TileCoding::getMinWeight();
}


const double ActionTileCoding::getAvgWeight( const bool abs ) const
{
  return TileCoding::getAvgWeight( abs );
}


const double ActionTileCoding::getMaxWeight() const
{
  return TileCoding::getMaxWeight();
}


void ActionTileCoding::setState()
{
  TileCoding::setState();
}


void ActionTileCoding::initEpisodeStateHistory()
{
  TileCoding::initEpisodeStateHistory();
}


void ActionTileCoding::valueEpisode( const double value )
{
  TileCoding::valueEpisode( value );
}


void ActionTileCoding::removeAction( const int actionIndex )
{
  bool removed = false;
  for ( std::list< Tilings* >::const_iterator t = tilingPtrList.begin();
        t != tilingPtrList.end(); ++t )
  {
    removed = static_cast< ActionTilings* >(*t)->removeAction( actionIndex );
    if ( removed == false ) {
      RL_LOGERROR( "Action could not be removed!" );
    }
  }
}


const int ActionTileCoding::save( FILE* file ) const
{
  return TileCoding::save( file );
}


const int ActionTileCoding::load( FILE* file )
{
  // falls schon Daten vorhanden sind --> Speicher freigeben und neu allozieren
  freeMemory();

  // Anzahl der Tilings
  unsigned int listSize = 0;
  if ( fread( &listSize, sizeof( unsigned int ), 1, file ) != 1 ) {
    RL_LOGERROR( "errno = " << errno << " --> " << strerror( errno ) );
    return -1;
  }

  // Tilings
  for ( unsigned int t = 0; t < listSize; ++t ) {
    tilingPtrList.push_back( new ActionTilings( statePtr, ACTIONS, file ) );
    numberOfTilings += (tilingPtrList.back())->getNumberOfTilings();
  }

  return 0;
}
