#ifndef __ACTION_TILINGS_H__
#define __ACTION_TILINGS_H__

#include "Tilings.h"

//class State;


class ActionTilings : public Tilings
{
  public:
    /**
    * Konstruktor
    * @param numberOfTilings - Anzahl der Tilings
    * @param varList - Liste mit den Daten der Variablen
    */
    ActionTilings( const int numberOfTilings, const std::list< Types::TILINGS_VAR_STRUCT >& varList );

    /**
    * Konstruktor zum Laden eines Tilings
    * @param aStatePtr - ein Pointer auf den Zustand
    * @param numberOfActions - die Anzahl der Aktionen
    * @param file - Filehandle
    */
    ActionTilings( State* aStatePtr, const int numberOfActions, FILE* file );

    /**
    * Destruktor
    */
    virtual ~ActionTilings();

    /**
    * Erzeugt das Tiling wirklich
    * @param aStatePtr - ein Pointer auf den Zustand
    * @param numberOfActions - die Anzahl der Aktionen
    */
    virtual void create( State* aStatePtr, const int numberOfActions );

    /**
    * Die Traces der aktivierten Features werden erhoeht.
    * @param VALUE - der Wert um den erhoeht wird
    * @param actionIndex - der Index der Aktion
    */
    virtual void accumulatingTraces( const double VALUE, const int actionIndex );

    /**
    * Die Traces der aktivierten Features werden gesetzt.
    * @param VALUE - der Wert auf den gesetzt wird
    * @param actionIndex - der Index der Aktion
    */
    virtual void replacingTraces( const double VALUE, const int actionIndex );

    /**
    * Falls keine EligibilityTraces verwendet werden, muss der Zustand gespeichert
    * werden, damit man weiss welche Gewichte geupdatet werden muessen.
    * --> die aktivierten Tiles speichern
    * @param actionIndex - der Index der Aktion
    */
    virtual void storeActiveTiles( const int actionIndex );

    /**
    * Liefert den Wert einer Aktion in einem Zustands mit den zuvor aktivierten Tiles.
    *@param actionIndex - der Index der Aktion
    * @return Wert der Aktion
    */
    virtual const double getActionValue( const int actionIndex ) const;

    /**
    * Speichert den Zustand in der Historie dieser Episode ab.
    * Dazu werden alle Indizees der aktiven Gewichte gespeichert.
    */
    virtual void storeEpisodeState( const int actionIndex );

    /**
    * Setzt den Wert fuer eine Aktion in allen Zustaenden.
    * Sinn: Soll optimistische/pessimistische Werte fuer einzelne Aktionen ermoeglichen,
            damit diese einen Vorteil/Nachteil erhalten.
      @param actionIndex - der Index der Aktion
      @param value - der Wert der Aktion
    */
    virtual void setActionValue( const int actionIndex, const double value );

    /**
    * Setzt den Wert fuer eine Aktion im aktuellen Zustand (aktivierte Tilings).
    * Sinn: Soll optimistische/pessimistische Werte fuer einzelne Aktionen ermoeglichen,
            damit diese einen Vorteil/Nachteil erhalten (um z.B. einzelne Aktionen in Zustaenden
            zu verhindern).
      @param actionIndex - der Index der Aktion
      @param value - der Wert der Aktion
    */
    virtual void setActiveActionValue( const int actionIndex, const double value );

    /**
    * Entfernt die Aktion aus dem ActionTiling und reduziert die Gesamtanzahl der Aktionen.
    * Achtung! EligibilityTraces werden geloescht, es gibt keine aktiven Tiles mehr und die Indexhistory wird geleert!
    * @param der Index der Aktion die entfernt werden soll
    * @return true, wenn erfolgreich; false sonst
    */
    virtual const bool removeAction( const int actionIndex );


  protected:
    /** Anzahl der Aktionen (kann leider nicht const sein wegen laden) */
    int ACTIONS;
};

#endif
