#include "Sarsa.h"
#include "Log.h"
#include "ActionValueFunction.h"


Sarsa::Sarsa( const std::string& fileName )
  : RL_PP( fileName )
{
  // Action-Zustands-Wertfunktion initialisieren --> auf 0
  avfPtr = 0;
}


Sarsa::~Sarsa()
{
  delete avfPtr;
}


void Sarsa::initEpisode()
{
  // und wieder eine Episode mehr
  ++episodes;
  // wir haben noch keinen Schritt gemacht
  steps = 0;

  if ( TRACES != NONE ) {
    // eligibility traces initialisieren (auf 0 setzen)
    avfPtr->initEligibilityTraces();
  }

  if ( avfPtr->getEpisodeStateHistory() == true ) {
    // die episodeStateHistory initialisieren (also leeren)
    avfPtr->initEpisodeStateHistory();
  }

  // in diesem Zustand starten wir
  setInitialState();

  // die ValueFunction muss den neuen Zustand uebernehmen
  avfPtr->setState();

  // Aktion waehlen
  switch ( ACTION_SELECTION ) {
    case GREEDY:
      // argmax Aktion
      value = avfPtr->setGreedyActionIndex();
      break;

    case EPSILON_GREEDY:
      if ( randomAction() == true ) {
        // zufaellige Aktion
        value = avfPtr->setRandomActionIndex();
      }
      else {
        // argmax Aktion
        value = avfPtr->setGreedyActionIndex();
      }
      break;

    case SOFTMAX:
      value = avfPtr->setSoftMaxActionIndex( TEMPERATURE );
      break;

    default:
      RL_LOGERROR( "Unknown ACTION_SELECTION " << ACTION_SELECTION );
  }
}


void Sarsa::step()
{
  // wieder ein Schritt mehr in dieser Episode
  ++steps;

  if ( TRACES == NONE ) {
    avfPtr->storeState();
  }
  else {
    // eligibility traces updaten
    avfPtr->updateEligibilityTraces( GAMMA, LAMBDA );

    // Traces
    switch ( TRACES )
    {
      case REPLACING:
        avfPtr->replacingTraces( TRACE_VALUE );
        break;

      case ACCUMULATING:
        avfPtr->accumulatingTraces( TRACE_VALUE );
        break;

      default:
        RL_LOGERROR( "Unknown Tracetype!" );
    }
  }

  // Aktion ausfuehren
  takeAction();

  // neuen Zustand ermitteln
  continueEpisode = observeState();

  // die ValueFunction muss den neuen Zustand uebernehmen
  avfPtr->setState();

  // reward bestimmen
  double reward = calculateReward();

  // Delta berechnen
  double delta = reward - value;

  // new Action
  if ( continueEpisode ) {
    // Aktion waehlen
    switch ( ACTION_SELECTION ) {
      case GREEDY:
        // argmax Aktion
        value = avfPtr->setGreedyActionIndex();
        break;
  
      case EPSILON_GREEDY:
        if ( randomAction() == true ) {
          // zufaellige Aktion
          value = avfPtr->setRandomActionIndex();
        }
        else {
          // argmax Aktion
          value = avfPtr->setGreedyActionIndex();
        }
        break;
  
      case SOFTMAX:
        value = avfPtr->setSoftMaxActionIndex( TEMPERATURE );
        break;
  
      default:
        RL_LOGERROR( "Unknown ACTION_SELECTION " << ACTION_SELECTION );
    }

    delta += GAMMA * value;
  }

  // Gewichte updaten
  avfPtr->updateWeights( ALPHA, delta );

  if ( continueEpisode ) {
    // den value nocheinmal berechnen --> nach dem Gewichtsupdate
    value = avfPtr->getActionValue( avfPtr->getActiveActionIndex() );
  }
}


const bool Sarsa::checkValueFunction() const
{
  return (avfPtr != 0);
}
