#include "State.h"

#include <cerrno>
#include <climits>


State::State()
{
  // type
  varType = UNDEF;
  // vars init
  vars = std::vector< VarValues* >( 0 );
}


State::State( FILE* file )
{
  // type
  varType = UNDEF;
  // vars init
  vars = std::vector< VarValues* >( 0 );
  // Laden
  load( file );
}


State::~State()
{
}


const unsigned int State::registerVariable( const std::string& name, const double minValue, const double maxValue )
{
  if ( vars.size() == 0 ) {
    varType = DOUBLE;
  }

  if ( varType != DOUBLE ) {
    RL_LOGERROR( "Cannot register Double-Variable!" );
    return UINT_MAX;
  }

  if ( minValue > maxValue ) {
    vars.push_back( new VarValuesDouble( name, maxValue, minValue ) );
  }
  else {
    vars.push_back( new VarValuesDouble( name, minValue, maxValue ) );
  }

  return (vars.size() - 1);
}


const unsigned int State::registerVariable( const std::string& name, const int minValue, const int maxValue )
{
  if ( vars.size() == 0 ) {
    varType = INT;
  }

  if ( varType != INT ) {
    RL_LOGERROR( "Cannot register Int-Variable!" );
    return UINT_MAX;
  }

  if ( minValue > maxValue ) {
    vars.push_back( new VarValuesInt( name, maxValue, minValue ) );
  }
  else {
    vars.push_back( new VarValuesInt( name, minValue, maxValue ) );
  }

  return (vars.size() - 1);
}


const unsigned int State::getVariableId( const std::string& name ) const
{
  for ( unsigned int v = 0; v < vars.size(); ++v ) {
    if ( vars[v]->getName() == name ) {
      return v;
    }
  }

  RL_LOGERROR( "No variable with name = " << name );
  return UINT_MAX;
}


const int State::save( FILE* file ) const
{
  // Anzahl der Variablen
  unsigned int varsSize = vars.size();
  if ( fwrite( &varsSize, sizeof( unsigned int ), 1, file ) != 1 ) {
    RL_LOGERROR( "errno = " << errno << "  -->  " << strerror( errno ) );
    return -1;
  }

  // Typ der Variablen speichern
  if ( fwrite( &varType, sizeof( TYPE ), 1, file ) != 1 ) {
    RL_LOGERROR( "errno = " << errno << "  -->  " << strerror( errno ) );
    return -1;
  }

  // name, min- und maxValue der Variablen
  int returnValue = 0;
  for ( std::vector< VarValues* >::const_iterator v = vars.begin(); v != vars.end(); ++v ) {
    returnValue = (*v)->save( file );
    if ( returnValue != 0 ) {
      return returnValue;
    }
  }

  return 0;
}


const int State::load( FILE* file )
{
  // vars leeren
  vars.clear();

  // Anzahl der Variablen
  unsigned int varsSize = 0;
  if ( fread( &varsSize, sizeof( unsigned int ), 1, file ) != 1 ) {
    RL_LOGERROR( "errno = " << errno << "  -->  " << strerror( errno ) );
    return -1;
  }

  if ( fread( &varType, sizeof( TYPE ), 1, file ) != 1 ) {
    RL_LOGERROR( "errno = " << errno << "  -->  " << strerror( errno ) );
    return -1;
  }

  // name, min- und maxValue der Variablen
  switch ( varType ) {
    case DOUBLE:
      for ( unsigned int i = 0; i < varsSize; ++i ) {
        vars.push_back( new VarValuesDouble( file ) );
      }
      break;

    case INT:
      for ( unsigned int i = 0; i < varsSize; ++i ) {
        vars.push_back( new VarValuesInt( file ) );
      }
      break;

    default:
      RL_LOGERROR( "Undef VarType: " << varType );
  }

  return 0;
}
