#include "StateNeuralNetwork.h"
#include "State.h"
#include "Utility.h"

#include <cerrno>
#include <cfloat>
#include <cmath>


StateNeuralNetwork::StateNeuralNetwork( State* aStatePtr, const int numberOfHiddenNeurons )
  : StateValueFunction( aStatePtr ),
    NeuralNetwork( aStatePtr, numberOfHiddenNeurons )
{
  // den Funktionspointer initialisieren
  ValueFunction::minValue = -0.5;
  ValueFunction::maxValue = 0.5;
  initWeightsFunctionPtr = &getRandomValue;
  
  create();
}


StateNeuralNetwork::StateNeuralNetwork( const std::string& fileName )
  : StateValueFunction( 0 ),
    NeuralNetwork()
{
  // Laden
  StateValueFunction::load( fileName );
}


StateNeuralNetwork::~StateNeuralNetwork()
{
}


void StateNeuralNetwork::create()
{
  INPUT_HIDDEN_ARRAY_SIZE = INPUT_LAYER_NEURONS * (HIDDEN_LAYER_NEURONS - 1); // -1 wegen Bias
  HIDDEN_OUTPUT_ARRAY_SIZE = HIDDEN_LAYER_NEURONS * OUTPUT_LAYER_NEURONS;
  INPUT_HIDDEN_OUTPUT_ARRAY_SIZE = INPUT_LAYER_NEURONS * (HIDDEN_LAYER_NEURONS - 1) * OUTPUT_LAYER_NEURONS; // -1 wegen Bias

  NeuralNetwork::allocateMemory();
  
  // init Skalierungsfaktoren
  for ( int i = 0; i < INPUT_LAYER_NEURONS - 1; ++i ) { // - Bias
    scaleFactor[i] = 1.0;
  }

  // Bias setzen
  setBias( 1.0, 1.0 );
}


void StateNeuralNetwork::initWeights( const double (*initFunctionPtr)() )
{
  NeuralNetwork::initWeights(initFunctionPtr);
}


void StateNeuralNetwork::initEligibilityTraces()
{
  NeuralNetwork::initEligibilityTraces();
}


void StateNeuralNetwork::updateWeights( const double ALPHA, const double DELTA )
{
  NeuralNetwork::updateWeights(ALPHA, DELTA, minWeightLimit, maxWeightLimit);
}


void StateNeuralNetwork::updateEligibilityTraces( const double GAMMA, const double LAMBDA )
{
  NeuralNetwork::updateEligibilityTraces(GAMMA, LAMBDA);
}


void StateNeuralNetwork::accumulatingTraces( const double VALUE )
{
  NeuralNetwork::accumulatingTraces(VALUE);
}


void StateNeuralNetwork::replacingTraces( const double VALUE )
{
  NeuralNetwork::replacingTraces(VALUE);
}


const double StateNeuralNetwork::getStateValue() const
{
  // wir haben nur ein OutputNeuron
  return NeuralNetwork::getOutputValue(0);
}


void StateNeuralNetwork::checkWeightLimit()
{
  NeuralNetwork::checkWeightLimit(minWeightLimit, maxWeightLimit);
}


const double StateNeuralNetwork::getMinWeight() const
{
  return NeuralNetwork::getMinWeight();
}


const double StateNeuralNetwork::getAvgWeight( const bool abs ) const
{
  return NeuralNetwork::getAvgWeight(abs);
}


const double StateNeuralNetwork::getMaxWeight() const
{
  return NeuralNetwork::getMaxWeight();
}


void StateNeuralNetwork::setState()
{
  NeuralNetwork::setState();
}


void StateNeuralNetwork::storeState()
{
  NeuralNetwork::storeState();
}


void StateNeuralNetwork::initEpisodeStateHistory()
{
  NeuralNetwork::initEpisodeStateHistory();
}


void StateNeuralNetwork::valueEpisode( const double value )
{
  NeuralNetwork::valueEpisode(value);
}


const int StateNeuralNetwork::save( FILE* file ) const
{
  return NeuralNetwork::save( file );
}


const int StateNeuralNetwork::load( FILE* file )
{
  // jetzt ist der StatePtr bereits geladen --> auch im LookupTable setzen
  NeuralNetwork::setStatePtr( ValueFunction::statePtr );
  return NeuralNetwork::load( file );
}
