#include "StateTileCoding.h"
#include "Log.h"
#include "StateTilings.h"

#include <cerrno>
#include <cfloat>


StateTileCoding::StateTileCoding( State* aStatePtr )
  : StateValueFunction( aStatePtr ),
    TileCoding()
{
  // den Funktionspointer initialisieren
  value = 0.0;
  initWeightsFunctionPtr = &getValue;
}


StateTileCoding::StateTileCoding( const std::string& fileName )
  : StateValueFunction( 0 ),
    TileCoding()
{
  // Laden
  StateValueFunction::load( fileName );
}


StateTileCoding::~StateTileCoding()
{
}


void StateTileCoding::initWeights( const double (*initFunctionPtr)() )
{
  TileCoding::initWeights( initFunctionPtr );
}


void StateTileCoding::initEligibilityTraces()
{
  TileCoding::initEligibilityTraces();
}


void StateTileCoding::updateWeights( const double ALPHA, const double DELTA )
{
  TileCoding::updateWeights( ALPHA, DELTA );
}


void StateTileCoding::updateEligibilityTraces( const double GAMMA, const double LAMBDA )
{
  TileCoding::updateEligibilityTraces( GAMMA, LAMBDA );
}


void StateTileCoding::registerTiling( StateTilings* stateTilingPtr )
{
  stateTilingPtr->create( statePtr );
  stateTilingPtr->initWeights( initWeightsFunctionPtr );
  tilingPtrList.push_back( stateTilingPtr );
  numberOfTilings += stateTilingPtr->getNumberOfTilings();
}


const double StateTileCoding::getStateValue() const
{
  double value = 0.0;
  for ( std::list< Tilings* >::const_iterator t = tilingPtrList.begin();
        t != tilingPtrList.end(); ++t )
  {
    value += static_cast< StateTilings* >(*t)->getStateValue();
  }
  return value;
}


void StateTileCoding::accumulatingTraces( const double VALUE )
{
  for ( std::list< Tilings* >::iterator t = tilingPtrList.begin();
        t != tilingPtrList.end(); ++t )
  {
    static_cast< StateTilings* >(*t)->accumulatingTraces( VALUE );
  }

  if ( episodeStateHistory ) {
    for ( std::list< Tilings* >::const_iterator t = tilingPtrList.begin();
          t != tilingPtrList.end(); ++t )
    {
      static_cast< StateTilings* >(*t)->storeEpisodeState();
    }
  }
}


void StateTileCoding::replacingTraces( const double VALUE )
{
  for ( std::list< Tilings* >::iterator t = tilingPtrList.begin();
        t != tilingPtrList.end(); ++t )
  {
    static_cast< StateTilings* >(*t)->replacingTraces( VALUE );
  }

  if ( episodeStateHistory ) {
    for ( std::list< Tilings* >::const_iterator t = tilingPtrList.begin();
          t != tilingPtrList.end(); ++t )
    {
      static_cast< StateTilings* >(*t)->storeEpisodeState();
    }
  }
}


void StateTileCoding::storeState()
{
  for ( std::list< Tilings* >::const_iterator t = tilingPtrList.begin();
        t != tilingPtrList.end(); ++t )
  {
    static_cast< StateTilings* >(*t)->storeActiveTiles();
  }

  if ( episodeStateHistory ) {
    for ( std::list< Tilings* >::const_iterator t = tilingPtrList.begin();
          t != tilingPtrList.end(); ++t )
    {
      static_cast< StateTilings* >(*t)->storeEpisodeState();
    }
  }
}


void StateTileCoding::checkWeightLimit()
{
  // Grenzen der Gewichte ueberpruefen
  if ( (minWeightLimit > -1.0 * DBL_MAX) || (maxWeightLimit < DBL_MAX) ) {
    for ( std::list< Tilings* >::iterator t = tilingPtrList.begin();
          t != tilingPtrList.end(); ++t )
    {
      (*t)->checkWeightLimit( minWeightLimit, maxWeightLimit );
    }
  }
}


const double StateTileCoding::getMinWeight() const
{
  return TileCoding::getMinWeight();
}


const double StateTileCoding::getAvgWeight( const bool abs ) const
{
  return TileCoding::getAvgWeight( abs );
}


const double StateTileCoding::getMaxWeight() const
{
  return TileCoding::getMaxWeight();
}


void StateTileCoding::setState()
{
  TileCoding::setState();
}


void StateTileCoding::initEpisodeStateHistory()
{
  TileCoding::initEpisodeStateHistory();
}


void StateTileCoding::valueEpisode( const double value )
{
  TileCoding::valueEpisode( value );
}


const int StateTileCoding::save( FILE* file ) const
{
  return TileCoding::save( file );
}


const int StateTileCoding::load( FILE* file )
{
  // falls schon Daten vorhanden sind --> Speicher freigeben und neu allozieren
  freeMemory();

  // Anzahl der Tilings
  unsigned int listSize = 0;
  if ( fread( &listSize, sizeof( unsigned int ), 1, file ) != 1 ) {
    RL_LOGERROR( "errno = " << errno << " --> " << strerror( errno ) );
    return -1;
  }

  // Tilings
  for ( unsigned int t = 0; t < listSize; ++t ) {
    tilingPtrList.push_back( new StateTilings( statePtr, file ) );
    numberOfTilings += (tilingPtrList.back())->getNumberOfTilings();
  }

  return 0;
}
