#ifndef __STATE_TILE_CODING_H__
#define __STATE_TILE_CODING_H__

#include "StateValueFunction.h"
#include "TileCoding.h"

class StateTilings;


class StateTileCoding : public StateValueFunction,
                        public TileCoding
{
  public:
    /**
    * Konstruktor
    * @param aStatePtr - Pointer auf den Zustand
    */
    StateTileCoding( State* aStatePtr );

    /**
    * Konstruktor zum Laden
    * @param fileName - Name der zu ladenden Datei
    */
    StateTileCoding( const std::string& fileName );

    /**
    * Destruktor
    */
    ~StateTileCoding();

    /**
    * Initialisiert die Gewichte
    * --> Diese Implementation ruft nur die Implementation aus TileCoding auf!
    * @param initFunctionPtr - Pointer auf die Funktion die zum Initialisieren bentzt wird
    */
    virtual void initWeights( const double (*initFunctionPtr)() );

    /**
    * Initialisieren der EligibilityTraces mit 0.0
    * --> Diese Implementation ruft nur die Implementation aus TileCoding auf!
    */
    virtual void initEligibilityTraces();

    /**
    * Updaten der Gewichte
    * --> Diese Implementation ruft nur die Implementation aus TileCoding auf!
    * @param ALPHA - die Lernrate
    * @param DELTA - die Aenderung des Gewichts
    */
    virtual void updateWeights( const double ALPHA, const double DELTA );

    /**
    * Updaten der EligibilityTraces
    * --> Diese Implementation ruft nur die Implementation aus TileCoding auf!
    * @param GAMMA - Discountrate
    * @param LAMBDA - TraceDecay
    */
    virtual void updateEligibilityTraces( const double GAMMA, const double LAMBDA );

    /**
    * Anmelden eines Tilings
    * @param stateTilingPtr - Pointer auf ein State-Tilingsobjekt
    */
    virtual void registerTiling( StateTilings* stateTilingPtr );

    /**
    * Liefert den Wert fuer den aktuellen Zustand.
    * @return der Wert der Aktion
    */
    virtual const double getStateValue() const;

    /**
    * Die Traces der aktivierten Features werden erhoeht.
    * @param VALUE - der Wert um den erhoeht wird
    */
    virtual void accumulatingTraces( const double VALUE );

    /**
    * Die Traces der aktivierten Features werden gesetzt.
    * @param VALUE - der Wert auf den gesetzt wird
    */
    virtual void replacingTraces( const double VALUE );

    /**
    * Falls keine EligibilityTraces verwendet werden, muss der Zustand gespeichert
    * werden, damit man weiss welche Gewichte geupdatet werden muessen.
    * Beim TileCoding werden dazu die aktivierten Tiles gespeichert.
    * --> Diese Implementation ruft nur die Implementation aus TileCoding auf!
    */
    virtual void storeState();

    /**
    * Ueberprueft ob die Gewichte in den Grenzen sind.
    * minWeight <= weight <= maxWeight
    */
    virtual void checkWeightLimit();

    /**
    * Liefert das minimale Gewicht zurueck.
    * --> Diese Implementation ruft nur die Implementation aus TileCoding auf!
    * @return das minimale Gewicht
    */
    virtual const double getMinWeight() const;

    /**
    * Liefert das durchschnittliche Gewicht zurueck.
    * --> Diese Implementation ruft nur die Implementation aus TileCoding auf!
    * @param abs - wenn true wird der Durchschnitt ueber den absoluten Betrag der Gewichte gebildet
    * @return das durchschnittliche Gewicht
    */
    virtual const double getAvgWeight( const bool abs ) const;

    /**
    * Liefert das maximale Gewicht zurueck.
    * --> Diese Implementation ruft nur die Implementation aus TileCoding auf!
    * @return das maximale Gewicht
    */
    virtual const double getMaxWeight() const;

    /**
    * Setzt den Zustand --> aktiviert die Tilings
    * --> Diese Implementation ruft nur die Implementation aus TileCoding auf!
    */
    virtual void setState();

    /**
    * Initialisiert die episodeStateHistory.
    * --> Diese Implementation ruft nur die Implementation aus TileCoding auf!
    */
    virtual void initEpisodeStateHistory();

    /**
    * Bewertet die gesamte Episode.
    * --> dafuer muss episodeStateHistory auf true gesetzt sein
    * --> Diese Implementation ruft nur die Implementation aus TileCoding auf!
    * @param value - dieser Wert wird auf alle Gewichte der Episode gerechnet
    */
    virtual void valueEpisode( const double value );


  protected:
    /**
    * Abspeichern
    * --> Diese Implementation ruft nur die Implementation aus TileCoding auf!
    * @param file - Filehandle
    * @return 0 bei Erfolg, -1 bei Fehler
    */
    virtual const int save( FILE* file ) const;

    /**
    * Laden
    * @param file - Filehandle
    * @return 0 bei Erfolg, -1 bei Fehler
    */
    virtual const int load( FILE* file );
};

#endif
