#include "StateTilings.h"
#include "Log.h"
#include "State.h"

#include <cmath>
#include <cfloat>


StateTilings::StateTilings( const int numberOfTilings, const std::list< Types::TILINGS_VAR_STRUCT >& varList )
  : Tilings( numberOfTilings, varList )
{
}


StateTilings::StateTilings( State* aStatePtr, FILE* file )
  : Tilings( aStatePtr, file )
{
}


StateTilings::~StateTilings()
{
}


void StateTilings::create( State* aStatePtr )
{
  // Pointer auf den Zustand
  statePtr = aStatePtr;
  // tiles
  for ( int d = 0; d < DIMENSIONS; ++d ) {
    tiles[d] = static_cast< int >( ceil( (statePtr->getMaxValueDbl( vars[d].id ) - statePtr->getMinValueDbl( vars[d].id )) / (vars[d].resolution * TILINGS) ) );
    if ( vars[d].tileType == Types::WRAP ) {
      tiles[d] += 1;
    }
  }

  // resolution neu setzen --> da nun die tiles ermittelt sind
  for ( int d = 0; d < DIMENSIONS; ++d ) {
    vars[d].resolution = (statePtr->getMaxValueDbl( vars[d].id ) - statePtr->getMinValueDbl( vars[d].id )) / ((tiles[d] - ((vars[d].tileType == Types::WRAP) ? 1 : 0)) * TILINGS);
  }

  // tileWidth
  for ( int d = 0; d < DIMENSIONS; ++d ) {
    tileWidth[d] = (statePtr->getMaxValueDbl( vars[d].id ) - statePtr->getMinValueDbl( vars[d].id )) / (tiles[d] - ((vars[d].tileType == Types::WRAP) ? 1 : 0));
  }

  // indexOffset
  for ( int d = 0; d < DIMENSIONS; ++d ) {
    indexOffset[d] = TILINGS;
    for ( int i = d + 1; i < DIMENSIONS; ++i ) {
      indexOffset[d] *= tiles[i];
    }
  }

  // tileOffset
  for ( int t = 0; t < TILINGS; ++t ) {
    for ( int d = 0; d < DIMENSIONS; ++d ) {
      tileOffset[t][d] = vars[d].resolution * t;
    }
  }

  // Weight anlegen
  ARRAY_SIZE = TILINGS;
  for ( int d = 0; d < DIMENSIONS; ++d ) {
    ARRAY_SIZE *= tiles[d];
  }
  // Speicher fuer weights holen
  weights = allocateMemoryWeights();
}


void StateTilings::accumulatingTraces( const double VALUE )
{
  if ( traceStorage == Types::ARRAY ) {
    if ( eTraces == 0 ) {
      RL_LOGERROR( "traceStorage == ARRAY and eTraces == 0"
                   << " --> Set traceStorage to MAP or call initEligibilityTraces before calling updateWeights!" );
      return;
    }

    for ( int t = 0; t < TILINGS; ++t ) {
      if ( activeTiles[t] != -1 ) {
        eTraces[ activeTiles[t] ] += VALUE;
      }
    }
  }
  else if ( traceStorage == Types::MAP ) {
    std::map< int, double >::iterator i;

    for ( int t = 0; t < TILINGS; ++t ) {
      if ( activeTiles[t] != -1 ) {
        i = eTracesMap.find( activeTiles[t] );
        if ( i == eTracesMap.end() ) {
          // Key existiert noch nicht --> anlegen
          eTracesMap[ activeTiles[t] ] = VALUE;
        }
        else {
          // Key existiert schon --> Wert aufaddieren
          i->second += VALUE;
        }
      }
    }
  }
  else {
    RL_LOGERROR( "Unknown traceStorage = " << traceStorage );
  }
}


void StateTilings::replacingTraces( const double VALUE )
{
  if ( traceStorage == Types::ARRAY ) {
    if ( eTraces == 0 ) {
      RL_LOGERROR( "traceStorage == ARRAY and eTraces == 0"
                   << " --> Set traceStorage to MAP or call initEligibilityTraces before calling updateWeights!" );
      return;
    }

    for ( int t = 0; t < TILINGS; ++t ) {
      if ( activeTiles[t] != -1 ) {
        eTraces[ activeTiles[t] ] = VALUE;
      }
    }
  }
  else if ( traceStorage == Types::MAP ) {
    for ( int t = 0; t < TILINGS; ++t ) {
      if ( activeTiles[t] != -1 ) {
        eTracesMap[ activeTiles[t] ] = VALUE;
      }
    }
  }
  else {
    RL_LOGERROR( "Unknown traceStorage = " << traceStorage );
  }
}


void StateTilings::storeActiveTiles()
{
  if ( storedActiveIndices == 0 ) {
    storedActiveIndices = new int[ TILINGS ];
  }

  memcpy( storedActiveIndices, activeTiles, sizeof( int ) * TILINGS );
}


const double StateTilings::getStateValue() const
{
  if ( weights != 0 ) {
    double value = 0.0;
    for ( int t = 0; t < TILINGS; ++t ) {
      if ( activeTiles[t] != -1 ) {
        value += weights[ activeTiles[t] ];
      }
    }
    return value;
  }
  else {
    RL_LOGERROR( "weights == 0 --> Maybe forgot to call create?" );
    return DBL_MAX;
  }
}


void StateTilings::storeEpisodeState()
{
  for ( int t = 0; t < TILINGS; ++t ) {
    if ( activeTiles[t] != -1 ) {
      episodeIndexHistorySet.insert( activeTiles[t] );
    }
  }
}
