#ifndef __STATE_TILINGS_H__
#define __STATE_TILINGS_H__

#include "Tilings.h"

class State;


class StateTilings : public Tilings
{
  public:
    /**
    * Konstruktor
    * @param numberOfTilings - Anzahl der Tilings
    * @param varList - Liste mit den Daten der Variablen
    */
    StateTilings( const int numberOfTilings, const std::list< Types::TILINGS_VAR_STRUCT >& varList );

    /**
    * Konstruktor zum Laden eines Tilings
    * @param aStatePtr - ein Pointer auf den Zustand
    * @param file - Filehandle
    */
    StateTilings( State* aStatePtr, FILE* file );

    /**
    * Destruktor
    */
    virtual ~StateTilings();

    /**
    * Erzeugt das Tiling wirklich
    * @param aStatePtr - ein Pointer auf den Zustand
    */
    virtual void create( State* aStatePtr );

    /**
    * Die Traces der aktivierten Features werden erhoeht.
    * @param VALUE - der Wert um den erhoeht wird
    */
    virtual void accumulatingTraces( const double VALUE );

    /**
    * Die Traces der aktivierten Features werden gesetzt.
    * @param VALUE - der Wert auf den gesetzt wird
    */
    virtual void replacingTraces( const double VALUE );

    /**
    * Falls keine EligibilityTraces verwendet werden, muss der Zustand gespeichert
    * werden, damit man weiss welche Gewichte geupdatet werden muessen.
    * --> die aktivierten Tiles speichern
    */
    virtual void storeActiveTiles();

    /**
    * Liefert den Wert des Zustands mit den zuvor aktivierten Tiles.
    * @return Wert des Zustands
    */
    virtual const double getStateValue() const;

    /**
    * Speichert den Zustand in der Historie dieser Episode ab.
    * Dazu werden alle Indizees der aktiven Gewichte gespeichert.
    */
    virtual void storeEpisodeState();
};

#endif
