#include "StateValueFunction.h"
#include "Log.h"
#include "State.h"

#include <cerrno>



StateValueFunction::StateValueFunction( State* aStatePtr )
  : ValueFunction( aStatePtr )
{
}


StateValueFunction::~StateValueFunction()
{
}


const int StateValueFunction::save( const std::string& fileName ) const
{
  FILE* file = fopen( fileName.c_str(), "w+" );

  if ( file == 0 ) {
    RL_LOGERROR( "errno = " << errno << " --> " << strerror( errno ) );
    return -1;
  }

  // State speichern
  if ( statePtr->save( file ) != 0 ) {
    RL_LOGERROR( "saving state failed" );
    return -1;
  }

  // ValueFunction speichern
  if ( save( file ) != 0 ) {
    RL_LOGERROR( "saving valuefunction failed" );
    return -1;
  }

  if ( fclose( file ) != 0 ) {
    RL_LOGERROR( "errno = " << errno << " --> " << strerror( errno ) );
    return -1;
  }

  return 0;
}


const int StateValueFunction::load( const std::string& fileName )
{
  FILE *file = fopen( fileName.c_str(), "r" );

  if ( file == 0 ) {
    RL_LOGERROR( "errno = " << errno << " --> " << strerror( errno ) );
    return -1;
  }

  // State laden
  if ( statePtr == 0 ) {
    statePtr = new State( file );
  }
  else if ( statePtr->load( file ) != 0 ) {
    RL_LOGERROR( "loading state failed" );
    return -1;
  }

  // ValueFunction laden
  if ( load( file ) != 0 ) {
    RL_LOGERROR( "loading valuefunction failed" );
    return -1;
  }

  if ( fclose( file ) != 0 ) {
    RL_LOGERROR( "errno = " << errno << " --> " << strerror( errno ) );
    return -1;
  }

  return 0;
}
