#include "TD.h"
#include "Log.h"
#include "StateValueFunction.h"


TD::TD( const std::string& fileName )
  : RL_PP( fileName )
{
  // Zustands-Wertfunktion initialisieren --> auf 0
  svfPtr = 0;
}


TD::~TD()
{
  delete svfPtr;
}


void TD::initEpisode()
{
  // und wieder eine Episode mehr
  ++episodes;
  // wir haben noch keinen Schritt gemacht
  steps = 0;

  if ( TRACES != NONE ) {
    // eligibility traces initialisieren (auf 0 setzen)
    svfPtr->initEligibilityTraces();
  }

  if ( svfPtr->getEpisodeStateHistory() == true ) {
    // die episodeStateHistory initialisieren (also leeren)
    svfPtr->initEpisodeStateHistory();
  }

  // in diesem Zustand starten wir
  setInitialState();

  // die ValueFunction muss den neuen Zustand uebernehmen
  svfPtr->setState();

  // Wert des Zustands
  value = svfPtr->getStateValue();
}


void TD::step()
{
  // wieder ein Schritt mehr in dieser Episode
  ++steps;

  if ( TRACES == NONE ) {
    svfPtr->storeState();
  }
  else {
    // eligibility traces updaten
    svfPtr->updateEligibilityTraces( GAMMA, LAMBDA );

    // Traces
    switch ( TRACES )
    {
      case REPLACING:
        svfPtr->replacingTraces( TRACE_VALUE );
        break;

      case ACCUMULATING:
        svfPtr->accumulatingTraces( TRACE_VALUE );
        break;

      default:
        RL_LOGERROR( "Unknown Tracetype!" );
    }
  }

  // Aktion ausfuehren
  takeAction();

  // neuen Zustand ermitteln
  continueEpisode = observeState();

  // die ValueFunction muss den neuen Zustand uebernehmen
  svfPtr->setState();

  // reward bestimmen
  double reward = calculateReward();

  // Delta berechnen
  double delta = reward - value;

  // new Action
  if ( continueEpisode ) {
    // argmax Aktion
    value = svfPtr->getStateValue();

    delta += GAMMA * value;
  }

  // Gewichte updaten
  svfPtr->updateWeights( ALPHA, delta );

  if ( continueEpisode ) {
    // den value nocheinmal berechnen --> nach dem Gewichtsupdate
    value = svfPtr->getStateValue();
  }
}


const bool TD::checkValueFunction() const
{
  return (svfPtr != 0);
}
