#ifndef __TILE_CODING_H__
#define __TILE_CODING_H__

#include "Types.h"
#include <list>

class Tilings;


class TileCoding
{
  public:
    /**
    * Konstruktor
    */
    TileCoding();

    /**
    * Destruktor
    */
    virtual ~TileCoding();

    /**
    * Initialisiert die Gewichte
    * @param initFunctionPtr - Pointer auf die Funktion die zum Initialisieren bentzt wird
    */
    virtual void initWeights( const double (*initFunctionPtr)() );

    /**
    * Initialisieren der EligibilityTraces mit 0.0
    */
    virtual void initEligibilityTraces();

    /**
    * Updaten der Gewichte
    * @param ALPHA - die Lernrate
    * @param DELTA - die Aenderung des Gewichts
    */
    virtual void updateWeights( const double ALPHA, const double DELTA );

    /**
    * Updaten der EligibilityTraces
    * @param GAMMA - Discountrate
    * @param LAMBDA - TraceDecay
    */
    virtual void updateEligibilityTraces( const double GAMMA, const double LAMBDA );

    /**
    * Setzt die Art wie die Traces gehalten werden sollen.
    */
    virtual void setTraceStorage( const Types::TRACE_STORAGE newTraceStorage );

    /**
    * Die Traces der aktivierten Features werden erhoeht.
    * @param VALUE - der Wert um den erhoeht wird
    */
    virtual void accumulatingTraces( const double VALUE ) = 0;

    /**
    * Die Traces der aktivierten Features werden gesetzt.
    * @param VALUE - der Wert auf den gesetzt wird
    */
    virtual void replacingTraces( const double VALUE ) = 0;

    /**
    * Liefert das minimale Gewicht zurueck.
    * @return das minimale Gewicht
    */
    virtual const double getMinWeight() const;

    /**
    * Liefert das durchschnittliche Gewicht zurueck.
    * @param abs - wenn true wird der Durchschnitt ueber den absoluten Betrag der Gewichte gebildet
    * @return das durchschnittliche Gewicht
    */
    virtual const double getAvgWeight( const bool abs ) const;

    /**
    * Liefert das maximale Gewicht zurueck.
    * @return das maximale Gewicht
    */
    virtual const double getMaxWeight() const;

    /**
    * Setzt den Zustand --> aktiviert die Tilings
    */
    virtual void setState();

    /**
    * Initialisiert die episodeStateHistory.
    */
    virtual void initEpisodeStateHistory();

    /**
    * Bewertet die gesamte Episode.
    * --> dafuer muss episodeStateHistory auf true gesetzt sein
    * @param value - dieser Wert wird auf alle Gewichte der Episode gerechnet
    */
    virtual void valueEpisode( const double value );


  protected:
    /**
    * Abspeichern
    * @param file - Filehandle
    * @return 0 bei Erfolg, -1 bei Fehler
    */
    virtual const int save( FILE* file ) const;

    /**
    * Laden
    * @param file - Filehandle
    * @return 0 bei Erfolg, -1 bei Fehler
    */
    virtual const int load( FILE* file ) = 0;

    /** Speicher freigeben */
    virtual void freeMemory();


    /** Liste der registrierten Tilings. */
    std::list< Tilings* > tilingPtrList;

    /** Anzahl der Tilings */
    int numberOfTilings;
};

#endif
