#ifndef __VAR_VALUES_DOUBLE_H__
#define __VAR_VALUES_DOUBLE_H__

#include "VarValues.h"

class VarValuesDouble : public VarValues
{
  public:
    /**
    * Konstruktor
    * @param name - Name der Variablen
    * @param aMinValue - der minimale Wert der Variablen
    * @param aMaxValue - der maximale Wert der Variablen
    */
    VarValuesDouble( const std::string& name, const double aMinValue, const double aMaxValue );

    /**
    * Konstruktor zum Laden.
    * @param file - Filehandle
    */
    VarValuesDouble( FILE* file );

    /**
    * Destruktor
    */
    virtual ~VarValuesDouble();

    /**
    * Abspeichern des Zustands
    * @param file - Filehandle
    * @return 0 bei Erfolg, -1 bei Fehler
    */
    virtual const int save( FILE* file ) const;

    /**
    * Laden des Zustands
    * @param file - Filehandle
    * @return 0 bei Erfolg, -1 bei Fehler
    */
    virtual const int load( FILE* file );


    /**
    * Liefert den value.
    */
    inline const double getValue() const;

    /**
    * Liefert den minValue.
    */
    inline const double getMinValue() const;

    /**
    * Liefert den maxValue.
    */
    inline const double getMaxValue() const;

    /**
    * Setzt den value.
    */
    inline void setValue( const double newValue );

    /**
    * Setzt den minValue.
    */
    inline void setMinValue( const double newMinValue );

    /**
    * Setzt den maxValue.
    */
    inline void setMaxValue( const double newMaxValue );


  protected:
    // der Wert
    double value;
    // minimaler Wert / untere Grenze
    double minValue;
    // maximaler Wert / obere Grenze
    double maxValue;
};


inline const double VarValuesDouble::getValue() const
{
  return value;
}

inline const double VarValuesDouble::getMinValue() const
{
  return minValue;
}

inline const double VarValuesDouble::getMaxValue() const
{
  return maxValue;
}

inline void VarValuesDouble::setValue( const double newValue )
{
  value = newValue;
}

inline void VarValuesDouble::setMinValue( const double newMinValue )
{
  minValue = newMinValue;
}

inline void VarValuesDouble::setMaxValue( const double newMaxValue )
{
  maxValue = newMaxValue;
}

#endif
