#include "VarValuesInt.h"
#include "Log.h"

#include <cerrno>


VarValuesInt::VarValuesInt( const std::string& name, const int aMinValue, const int aMaxValue )
  : VarValues( name ),
    minValue( aMinValue ),
    maxValue( aMaxValue )
{
  // init
  value = (minValue + maxValue) / 2;
}


VarValuesInt::VarValuesInt( FILE* file )
  : VarValues( "" )
{
  load( file );
}


VarValuesInt::~VarValuesInt()
{
}


const int VarValuesInt::save( FILE* file ) const
{
  // name
  unsigned int stringSize = name.size();
  if ( fwrite( &stringSize, sizeof( unsigned int ), 1, file ) != 1 ) {
    RL_LOGERROR( "errno = " << errno << "  -->  " << strerror( errno ) );
    return -1;
  }
  if ( fwrite( name.c_str(), sizeof( char ), stringSize, file ) != static_cast< size_t >( stringSize ) ) {
    RL_LOGERROR( "errno = " << errno << "  -->  " << strerror( errno ) );
    return -1;
  }
  // minValue
  if ( fwrite( &(minValue), sizeof( int ), 1, file ) != 1 ) {
    RL_LOGERROR( "errno = " << errno << "  -->  " << strerror( errno ) );
    return -1;
  }
  // maxValue
  if ( fwrite( &(maxValue), sizeof( int ), 1, file ) != 1 ) {
    RL_LOGERROR( "errno = " << errno << "  -->  " << strerror( errno ) );
    return -1;
  }

  return 0;
}


const int VarValuesInt::load( FILE* file )
{
  // name
  unsigned int stringSize = 0;
  if ( fread( &stringSize, sizeof( unsigned int ), 1, file ) != 1 ) {
    RL_LOGERROR( "errno = " << errno << "  -->  " << strerror( errno ) );
    return -1;
  }
  char* charPtr = new char[ stringSize ];
  if ( fread( charPtr, sizeof( char ), stringSize, file ) != static_cast< size_t >( stringSize ) ) {
    RL_LOGERROR( "errno = " << errno << "  -->  " << strerror( errno ) );
    return -1;
  }
  name = std::string( charPtr, stringSize );
  delete[] charPtr;

  // minValue
  if ( fread( &minValue, sizeof( int ), 1, file ) != 1 ) {
    RL_LOGERROR( "errno = " << errno << "  -->  " << strerror( errno ) );
    return -1;
  }
  // maxValue
  if ( fread( &maxValue, sizeof( int ), 1, file ) != 1 ) {
    RL_LOGERROR( "errno = " << errno << "  -->  " << strerror( errno ) );
    return -1;
  }

  // init
  value = (minValue + maxValue) / 2;

  return 0;
}
