#ifndef __VAR_VALUES_INT_H__
#define __VAR_VALUES_INT_H__

#include "VarValues.h"

class VarValuesInt : public VarValues
{
  public:
    /**
    * Konstruktor
    * @param name - Name der Variablen
    * @param aMinValue - der minimale Wert der Variablen
    * @param aMaxValue - der maximale Wert der Variablen
    */
    VarValuesInt( const std::string& name, const int aMinValue, const int aMaxValue );

    /**
    * Konstruktor zum Laden.
    * @param file - Filehandle
    */
    VarValuesInt( FILE* file );

    /**
    * Destruktor
    */
    virtual ~VarValuesInt();

    /**
    * Abspeichern des Zustands
    * @param file - Filehandle
    * @return 0 bei Erfolg, -1 bei Fehler
    */
    virtual const int save( FILE* file ) const;

    /**
    * Laden des Zustands
    * @param file - Filehandle
    * @return 0 bei Erfolg, -1 bei Fehler
    */
    virtual const int load( FILE* file );


    /**
    * Liefert den value.
    */
    inline const int getValue() const;

    /**
    * Liefert den minValue.
    */
    inline const int getMinValue() const;

    /**
    * Liefert den maxValue.
    */
    inline const int getMaxValue() const;

    /**
    * Setzt den value.
    */
    inline void setValue( const int newValue );

    /**
    * Setzt den minValue.
    */
    inline void setMinValue( const int newMinValue );

    /**
    * Setzt den maxValue.
    */
    inline void setMaxValue( const int newMaxValue );


  protected:
    // der Wert
    int value;
    // minimaler Wert / untere Grenze
    int minValue;
    // maximaler Wert / obere Grenze
    int maxValue;
};


inline const int VarValuesInt::getValue() const
{
  return value;
}

inline const int VarValuesInt::getMinValue() const
{
  return minValue;
}

inline const int VarValuesInt::getMaxValue() const
{
  return maxValue;
}

inline void VarValuesInt::setValue( const int newValue )
{
  value = newValue;
}

inline void VarValuesInt::setMinValue( const int newMinValue )
{
  minValue = newMinValue;
}

inline void VarValuesInt::setMaxValue( const int newMaxValue )
{
  maxValue = newMaxValue;
}

#endif
