#include "Watkins.h"
#include "Log.h"
#include "ActionValueFunction.h"


Watkins::Watkins( const std::string& fileName )
  : RL_PP( fileName )
{
  // Action-Zustands-Wertfunktion initialisieren --> auf 0
  avfPtr = 0;
}


Watkins::~Watkins()
{
  delete avfPtr;
}


void Watkins::initEpisode()
{
  // und wieder eine Episode mehr
  ++episodes;
  // wir haben noch keinen Schritt gemacht
  steps = 0;

  if ( TRACES != NONE ) {
    // eligibility traces initialisieren (auf 0 setzen)
    avfPtr->initEligibilityTraces();
  }

  if ( avfPtr->getEpisodeStateHistory() == true ) {
    // die episodeStateHistory initialisieren (also leeren)
    avfPtr->initEpisodeStateHistory();
  }

  // in diesem Zustand starten wir
  setInitialState();

  // die ValueFunction muss den neuen Zustand uebernehmen
  avfPtr->setState();

  // argmax Aktion
  value = avfPtr->setGreedyActionIndex();
}


void Watkins::step()
{
  // wieder ein Schritt mehr in dieser Episode
  ++steps;

  // Aktion waehlen
  switch ( ACTION_SELECTION ) {
    case GREEDY:
      // argmax Aktion --> schon gewaehlt
      if ( TRACES != NONE ) {
        // eligibility traces updaten
        avfPtr->updateEligibilityTraces( GAMMA, LAMBDA );
      }
      break;

    case EPSILON_GREEDY:
      if ( randomAction() == true ) {
        // zufaellige Aktion
        value = avfPtr->setRandomActionIndex();

        if ( TRACES != NONE ) {
          // eligibility traces auf 0 setzen
          avfPtr->initEligibilityTraces();
        }
      }
      else {
        if ( TRACES != NONE ) {
          // eligibility traces updaten
          avfPtr->updateEligibilityTraces( GAMMA, LAMBDA );
        }
      }
      break;

    case SOFTMAX:
      {
        // die alte Aktion --> argmax
        int oldActionIndex = avfPtr->getActiveActionIndex();
        // neue Aktion ermitteln
        value = avfPtr->setSoftMaxActionIndex( TEMPERATURE );
  
        if ( TRACES != NONE ) {
          if ( oldActionIndex == avfPtr->getActiveActionIndex() ) {
            // argmax == neue Aktion
            // eligibility traces updaten
            avfPtr->updateEligibilityTraces( GAMMA, LAMBDA );
          }
          else {
            // eligibility traces auf 0 setzen
            avfPtr->initEligibilityTraces();
          }
        }
      }
      break;

    default:
      RL_LOGERROR( "Unknown ACTION_SELECTION " << ACTION_SELECTION );
  }


  if ( TRACES == NONE ) {
    avfPtr->storeState();
  }
  else {
    // Traces
    switch ( TRACES )
    {
      case REPLACING:
        avfPtr->replacingTraces( TRACE_VALUE );
        break;

      case ACCUMULATING:
        avfPtr->accumulatingTraces( TRACE_VALUE );
        break;

      default:
        RL_LOGERROR( "Unknown Tracetype!" );
    }
  }


  // Aktion ausfuehren
  takeAction();

  // neuen Zustand ermitteln
  continueEpisode = observeState();

  // die ValueFunction muss den neuen Zustand uebernehmen
  avfPtr->setState();

  // reward bestimmen
  double reward = calculateReward();

  // Delta berechnen
  double delta = reward - value;

  // new Action
  if ( continueEpisode ) {
    // argMax waehlen --> Greedy
    value = avfPtr->setGreedyActionIndex();
    delta += GAMMA * value;
  }

  // Gewichte updaten
  avfPtr->updateWeights( ALPHA, delta );

  if ( continueEpisode ) {
    // den value nocheinmal berechnen --> nach dem Gewichtsupdate
    value = avfPtr->getActionValue( avfPtr->getActiveActionIndex() );
  }
}


const bool Watkins::checkValueFunction() const
{
  return (avfPtr != 0);
}
